# -*- Mode: perl -*-
# file: ElegansSubs.pm
# C. elegans browsing subroutines for use at Sanger & Montpellier

package ElegansSubs;
use strict 'vars';

use CGI qw(:standard);
use AceSubs qw(AceHeader Style Url Object2URL Configuration resolveUrl DB_Name);
use GeneSubs 'NCBI','PROTEOME','PUBMED';
use vars qw/@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS %BIB_PATTERNS/;

require Exporter;
@ISA = Exporter;

@EXPORT = qw(
	     print_top print_bottom print_warning
	    );
@EXPORT_OK = qw(%BIB_PATTERNS get_interpolated_position print_refs
		find_position print_expression_pattern GenBankLink
		TypeSelector Banner Header Footer 
		WBG CGC
		SEQUENCE_INFO GENE_INFO ABSTRACT_INFO ALLELE_INFO
		STRAIN_INFO MAP_REPORT MAP_DISPLAY SEQ_DISPLAY 
	     );
%EXPORT_TAGS = ('biblio'  => [qw(%BIB_PATTERNS WBG CGC ABSTRACT_INFO print_refs)],
		'sequence'=> [qw(get_interpolated_position print_expression_pattern
				 SEQUENCE_INFO SEQ_DISPLAY)],
		'locus'   => [qw(GENE_INFO ALLELE_INFO MAP_REPORT MAP_DISPLAY STRAIN_INFO)]
		 );

# Foreign URLs
use constant WBG         => 'http://elegans.swmed.edu/wli/';
use constant CGC         => 'http://www.sanger.ac.uk/Projects/C_elegans/CGC/';

# custom constants
use constant GENE_INFO       => 'gene';
use constant SEQUENCE_INFO   => 'sequence';
use constant ABSTRACT_INFO   => 'biblio';

# constants holding generic browsing URLS
use constant ALLELE_INFO     => 'tree?class=Allele';
use constant STRAIN_INFO     => 'tree?class=Strain';
use constant MAP_REPORT      => 'nearby_genes';  
use constant MAP_DISPLAY     => 'pic?class=Locus';
use constant SEQ_DISPLAY     => 'sequence';

# boilerplate for the top of the page
sub print_top {
  my ($object,$class,$title,@additional_header_stuff) = @_;
  $class ||= $object->class if defined $object;
  AceHeader();
  $title ||= $object ? "$class Report for: $object" : "$class Report" ;
  print start_html (
		    '-Title'   => $title,
		    '-Style'   => Style(),
		    @additional_header_stuff,
		    );
  print Banner();
  print TypeSelector($object,$class) if defined $object;
  print ElegansSubs::Header();
  print h1($title) if defined $object;
}

# boilerplate for the bottom of the page
sub print_bottom {
  print p(),ElegansSubs::Footer();
}

sub print_warning {
  my ($name,$class) = @_;
  print p(font({-color => 'red'},
	       "The $class named \"$name\" is not found in the database."));
}

# Where to find the NCBI/Genbank pages
sub GenBankLink {
  my $text = shift;
  $text =~ s!([PN])ID:\s?[gG]?(\d+)
            !CGI::a({-href=>NCBI . "?db=\L$1\E&form=1&field=Sequence+ID&term=$2"},$&)
	    !xeg;
  $text;
}

# Bibliographic stuff
%BIB_PATTERNS =(
		'ALL' => ['All Papers','.'],
		'WBG' => ['Worm Breeders Gazette Papers','^\[wbg'],
		'CGC' => ['CGC Bibliography','^\[cgc'],
		'WMA' => ['Worm Meeting Abstracts','^\[[a-z]{0,2}wm\d']
	       );	

my @BANNERS;

sub Header {
  my $width = Configuration->Pagewidth;
#  return br() . qq(<table align=left width=$width><tr><td align=left>);
  return br();
}

sub Footer {
 my $end_table = <<END;
</td>
</tr>
</table><br clear=all>
END
;
   $end_table = br({-clear=>'all'});
  return join '',Configuration()->My_footer, AceSubs::Footer(), $end_table, end_html();
}

# get the interpolated position of a sequence on the genetic map
# returns ($chromosome, $position)
# position is in genetic map coordinates
sub get_interpolated_position {
    my ($db,$obj) = @_;
    
    my ($full_name,$chromosome,$position);
    my $o = $obj ;
    while ($o) {
	# try to get the map position using the Sanger model first
	my $m = $o->get('Interpolated_gmap');
	if ($m) {
	    ($chromosome,$position) = $m->right->row;
	    last if $chromosome;
	} else {
	    # otherwise try to get Jean's "Full_name" tag
	    ($full_name) = $o->get('Full_name') ;
	    last if $full_name;
	}
	my $src = $o->Source;
	# correct for missing source fields in some records
	$src = $db->fetch(Sequence=>$1)
	    if !$src && $o=~/^(\w+)\.\d+/;
	$o = $src;
    }
    return ($chromosome,$position) if $chromosome;
    return undef unless $full_name;
    return $full_name =~ /\(([IVX]+):([\-0-9.]+)/; 
}

# find position of a sequence in genome coordinates and return:
#     (start, end, reference sequence)
# the reference sequence will usually (but not always!) be a chromosome
sub find_position {
  my ($DB,$seq) = @_;
  return undef unless my $s = $DB->fetch(Sequence=>$seq);
  my ($abs_offset,$length,$prev) = (1,0);
  
#  for ($prev=$s, my $o=get_source($s); $o; $prev=$o,$o=get_source($o)) {
  for ($prev=$s, my $o=$s->Source; $o; $prev=$o,$o=$o->Source) {
    my @subs = $o->get('Subsequence');
    my ($seq) = grep $prev eq $_,@subs;
    $length ||= $seq->right(2) - $seq->right(1) + 1;
    $abs_offset += $seq->right-1;    # offset to beginning of sequence
  }
  
  return ($abs_offset,$abs_offset+$length-1,$prev);
}

sub print_refs {
    my ($obj,$refs) = @_;
    my $n = CGI::escape($obj->name);
    my $c = CGI::escape($obj->class);
    print hr,h4({-class=>'heading'},'Bibliography:');
    print p({-class=>'note'},'No bibliography available in Wormbase')
      unless $refs and @$refs;

    my @cgc = grep (/$BIB_PATTERNS{'CGC'}->[1]/,@$refs);
    my @wbg = grep (/$BIB_PATTERNS{'WBG'}->[1]/,@$refs);
    my @wm  = grep (/$BIB_PATTERNS{'WMA'}->[1]/,@$refs);
    my @items;
    push (@items,a({-href=>Url(ABSTRACT_INFO,"name=$n&category=CGC&type=$c") },
		   "Papers in CGC Bibliography (" . scalar(@cgc).")")) if @cgc;
    push (@items,a({-href=>Url(ABSTRACT_INFO,"name=$n&category=WBG&type=$c") },
		   "Papers in Worm Breeders Gazette (" . scalar(@wbg) . ")")) if @wbg;
    push (@items,a({-href=>Url(ABSTRACT_INFO,"name=$n&category=WMA&type=$c") },
		   "Worm Meeting Abstracts (" . scalar(@wm) . ")")) if @wm;
    push @items,a({-href=>PUBMED . $obj},"Search PubMed for references");
    print ul(
	     li(\@items),
	     );
}

sub print_expression_pattern {
  my $gene = shift;
  my $pattern = $gene->Expr_pattern(1);
  my $class = $gene->class;
  
  print hr,h3({-class=>'heading'},'Expression Pattern:');
  unless ($pattern) {
    print p({-class=>'note'},
	    "Either no expression pattern has been described for this \L$class\E or it has not yet been entered into WormBase.");
    return;
  }
  print p(
	  {-class=>'smalldescription'},
	  $pattern->Pattern,
	  $pattern->Type ?  ( "(Assay: ". join(' ',$pattern->Type(1)->row) . ")")  : ''
	  );
  print p(
	  a({-href=>Object2URL($pattern)},'More information on this pattern')
	 );
}

# Choose a set of displayers based on the type.
sub TypeSelector {
    my ($name,$class) = @_;
    $name = "$name" if ref($name);  # fix obscure bug in escape
    my ($n,$c) = (CGI::escape($name),CGI::escape($class));
    my @rows;

    # add the special displays
    my @displays       = Configuration()->class2displays($class);
    my @basic_displays = Configuration()->class2displays('default');
    @basic_displays    = SiteConfig->getConfig(DEFAULT_DATABASE)->class2displays('default') 
      unless @basic_displays;

    my $display = url(-absolute=>1);

    foreach (@displays,@basic_displays) {
 	my ($url,$icon,$label) = @{$_}{qw/url icon label/};
	next unless $url;
	my $u = resolveUrl($url,"name=$n&class=$c");
	$url =~ s/\#.*$//;

	my $active = $url !~ /$display/;
	my $cell = $active ? a({-href=>$u,-target=>'_top'},$label)
	                   : font({-color=>'red'},$label);
	push (@rows,td({-align=>'CENTER',-class=>'small'},$cell));
      }

    my $width = Configuration->Pagewidth || 660;
    return table({-width=>$width,
		  -border=>'1', cellpadding=>'4', Cellspacing=>0, bgcolor=>"#fffcc"},
		   Tr({-valign=>'bottom'},@rows));
}

sub Banner {
  my $config = Configuration();
  my $dbname = DB_Name();

  return unless my $searches = $config->Searches;
  return unless my $banners  = $config->Banners;
  my $text_banner            = $config->Banner;
  my $size                   = $config->Banner_size;
  my $banner;

  # first select a random banner and turn it into an HTML <IMG>
  unless (@BANNERS) {
    my $physical = $ENV{MOD_PERL} ? Apache->request->lookup_uri($banners)->filename
                                  : "$ENV{DOCUMENT_ROOT}/$banners";
    opendir D,$physical or die "Couldn't open directory $physical: $!";
    @BANNERS = map {"$banners/$_"} grep {-f "$physical/$_" } grep {!/^\./} readdir(D);
    closedir D;
  }
  return unless @BANNERS;
  $banner = $BANNERS[rand @BANNERS];
  my $banner_image = $size ? img({-src=>$banner,-alt=>$text_banner,
				  -width=>$size->[0],-height=>$size->[1]})
                           : img({-src=>$banner,-alt=>$text_banner});

  # next select the correct search script
  my @searches = @{$searches};
  my $self = url(-relative=>1);
  my @row;
  foreach (@searches) {
    my ($name,$url,$on,$off,$size) = @{$config->searches($_)}{qw/name url onimage 
								offimage size/};
    my $active = $url =~ /\b$self\b/;
    my $image = $active ? $on : $off;

    # replace the url with a cookie, if one is defined
    my $cookie_name = "SEARCH_${dbname}_${_}";
    my $query_string = cookie($cookie_name);
    $url .= "?$query_string" if $query_string;

    push @row,a({-href=>$url},img({-src=>$image,-border=>0,
				   -width=>$size->[0],-height=>$size->[1],
				   -alt=>$name}));
  }

  my ($home_icon,$hbs)    = @{$config->Home_button};
  my ($home,$label) = @{$config->Home};

  my $width = Configuration->Pagewidth || 660;
  return table({-border=>0,cellpadding=>2, Cellspacing=>0,Width=>$width+4},
	       Tr(td({-align=>'CENTER'},\@row)),
	       Tr(td({-align=>'LEFT',-colspan=>scalar(@searches)},
		     $banner_image."<nobr>".
		     a({-href=>$home},
		       img ({-src=>$home_icon,-border=>0,
			     -width=>$hbs->[0],
			     -height=>$hbs->[1],
			     -alt=>$label})),
		     ))
	      );
}

1;
