package Chromosome;

use strict;
use Ace;
use Band;
use GD 'gdSmallFont';
use Contig;

use vars qw($VERSION $PNG);
$VERSION = '1.00';
$PNG = $GD::VERSION > 1.19;

use constant DEFAULT_WIDTH  => 500;
use constant DEFAULT_HEIGHT => 300;
use constant LEFT => 80; # coord of left side of chromosome
use constant I_WIDTH  => 30; # width of chromosome, invariant
use constant I_RADIUS => I_WIDTH / 2; # radius of rounded ends
use constant H_NAME   => 20;  # distance from left side of idiogram to column of band names
use constant TOP_MARG => 10; #set to 0 for default. Gets passed as $top_marg
use constant BOTTOM_MARG => 50;

sub new {
  my $class      = shift;
  my $cyto_map   = shift || die "must provide a cytogenetic map Ace object";
  my $db = shift;
  my ($chrom) = $cyto_map =~ /Chrom(\w+)/;
  my ($width,$height) = @_;
  $width  ||= DEFAULT_WIDTH;
  $height ||= DEFAULT_HEIGHT;

  # basic image stuff. "BOTTOM_MARG" adds some extra space at bottom
  my $pic = GD::Image::Chrom->new($width,$height+BOTTOM_MARG); 
  my %colors;
  @colors{qw(white black grey ltblue red)} = (
					      $pic->colorAllocate(255,255,255),
					      $pic->colorAllocate(0,0,0),
					      $pic->colorAllocate(200,200,200),
					      $pic->colorAllocate(135,206,250),
					      $pic->colorAllocate(255,0,0)
					     );
  
  my ($trunc_top,$trunc_bottom);
  # sort bands by their position on the map (not necessarily symbolically)
  my @bands = sort {$a->right <=> $b->right} $cyto_map->Band;
  die "nothing contained within interval" unless @bands;

  # turn the whole thing into a preparsed list using the Band class
  my @parsed_bands;
  for my $band (@bands) {
    my $name = new Band $band;
    my ($start,$end,$color) = $band->row(1);
    push @parsed_bands,[$name,$start,$end,$color eq 'black' ? 'ltblue' : $color];
  }
  
  my $scale = $class->calculate_scale($height,\@parsed_bands,0,$#parsed_bands);

  # find where the centromere is supposed to go (may be acrocentric)
  my $centromere = 0;
  for (my $i = 0; $i<@parsed_bands; $i++) {
    if ($parsed_bands[$i][0] =~ /q/) {
      $centromere = $i-1;
      last;
    }
  }

  return bless {
		gd          => $pic,
		colors      => \%colors,
		bands       => \@parsed_bands,
		band_start  => 0,
		band_end    => $#parsed_bands,
		centromere  => $centromere,
		width       => $width,
		height      => $height,
		scale       => $scale,
	    db          => $db,
        chrom       => $chrom
	       },$class;
}

# restrict range of the display (zoom)
sub restrict {
  my $self = shift;
  my ($start,$end) = @_;  #starting and ending band symbols
  die "call with start and end bands names" unless $start || $end;

  my $bands = $self->{'bands'};
  $start ||= $bands->[0][0];
  $end   ||= $bands->[$#{$bands}][0];
  chomp $start;
  chomp $end;

  die "start band must be less than end band" unless new Band($start) <= new Band($end);
  
#  my ($i1,$i2);
  my $i1 = 0;
  my $i2 = 0;
  for ($i1=0; $start > $bands->[$i1][0]; $i1++) { }
  for ($i2=0; $bands->[$i2][0] < $end; $i2++) { }
  die "programmer error $i1 < $2 not!" unless $i1 <= $i2;
  $self->{'band_start'} = $i1;
  $self->{'band_end'} = $i2;

  $self->calculate_scale();
}

sub calculate_scale {
  my $self = shift;
  my ($height,$bands,$start,$end) = @_;
  if (ref($self)) {
    $height ||= $self->{'height'};
    $bands ||= $self->{'bands'};
    $start ||= $self->{'band_start'};
    $end   ||= $self->{'band_end'};
  }
  my $scale = $height/($bands->[$end][2] - $bands->[$start][1]);
  $self->{'scale'} = $scale if ref($self);
  return $scale;
}

# retrieve or change size
sub size {
  my $self = shift;
  my ($x,$y) = @_;
  if (defined($x) || defined($y)) {
    $x ||= $self->{width};
    $y ||= $self->{height};
    @{$self}{qw(width height)} = ($x,$y);
    my $newpic = GD::Image::Chrom->new($x,$y);
    $newpic->paletteCopy($self->{gd});
    $self->{gd} = $newpic;
    $self->calculate_scale;
  }
  return $self->{gd}->getBounds;
}

# get the band coords for image map
sub coords {
		my $self 	   = shift;
		my $centromere = $self->centromere;
		my $start      = $self->start;
		my $end        = $self->end;
		my $bands      = $self->bands;
		my $pic        = $self->gd;
		my $left = LEFT;
		my $right = LEFT + I_WIDTH;
		my %coords = ();
		my $bound = $end < $centromere ? $end : $centromere;
		for (my $i = $start; $i <= $bound; $i++) {
				my ($name,$y1,$y2) = @{$bands->[$i]};
				$coords{$name} = [ $self->d2p($y1),$self->d2p($y2),$left,$right ];
		}
		$bound = $start > ($centromere + 1) ? $start : ($centromere + 1);
		for (my $i=$bound; $i <= $end; $i++) {
				my ($name,$y1,$y2) = @{$bands->[$i]};
				$coords{$name} = [ $self->d2p($y1),$self->d2p($y2), $left,$right ];
		}

		my $p_contig = Contig::contig( $bands, $start, $end, $self->{chrom}, $self->{db}, DEFAULT_WIDTH, DEFAULT_HEIGHT, TOP_MARG );


		return \%coords, $p_contig;
}


sub start       { $_[0]->{'band_start'}  } 
sub end         { $_[0]->{'band_end'}    } 
sub centromere  { $_[0]->{'centromere'}  } 
sub flat_top    { $_[0]->{'band_start'} != 0  }
sub flat_bottom { $_[0]->{'band_end'} != $#{$_[0]->{'bands'}}  }
sub bands       { $_[0]->{'bands'}       }
sub colors      { $_[0]->{'colors'}      }

sub d2p {
  return TOP_MARG + int(($_[1]-$_[0]->{bands}->[$_[0]->{'band_start'}][1]) * $_[0]->{'scale'} + 0.5); 
}

sub gd {
  return $_[0]->{'gd'};
}

sub png {
  return $_[0]->gd->png;
}

sub gif {
  return $_[0]->gd->png;
}

sub draw {
  my $self = shift;
  my $labels = shift;

  my $centromere = $self->centromere;
  my $start      = $self->start;
  my $end        = $self->end;
  my $bands      = $self->bands;
  my $trunc_top  = $self->flat_top;
  my $trunc_bottom  = $self->flat_bottom;
  my $pic        = $self->gd;
  my $colors     = $self->colors;
  my $black      = $colors->{black};
  
  my $left = LEFT;
  my $right = $left + I_WIDTH;
  my $center = $left + I_RADIUS;
  my $height = $self->{height};

  my @p_arm = ($left,TOP_MARG,$left+I_WIDTH,$self->d2p($bands->[$centromere][2]));
  my @q_arm = ($left,$self->d2p($bands->[$centromere][$centromere ? 2 : 1]),
	       $left+I_WIDTH,$height+TOP_MARG);

  $trunc_top = 0 if $start == ($centromere + 1);


  if ($centromere >= $start && $centromere < $end) {
    $pic->roundrect(@p_arm,$black,$trunc_top,0);
    $pic->roundrect(@q_arm,$black,0,$trunc_bottom);
  } else {
    $pic->roundrect($left,TOP_MARG,$left+I_WIDTH,$self->d2p($bands->[$end][2]),
		    $black,$trunc_top,$trunc_bottom);
  }

  #----- draw each band in candystripe fashion starting with the P band
  my $bound = $end < $centromere ? $end : $centromere;
  for (my $i = $start; $i <= $bound; $i++) {
    my ($name,$y1,$y2,$color) = @{$bands->[$i]};
    $pic->fillband(@p_arm,$self->d2p($y1),$self->d2p($y2),$colors->{$color},$trunc_top,0,TOP_MARG);
  }
  $bound = $start > ($centromere + 1) ? $start : ($centromere + 1);
  for (my $i=$bound; $i <= $end; $i++) {
    my ($name,$y1,$y2,$color) = @{$bands->[$i]};
	$pic->fillband(@q_arm,$self->d2p($y1),$self->d2p($y2),$colors->{$color},0,$trunc_bottom,TOP_MARG);
  }

  #------------------------------ call Contig::contig
  my $p_contig = Contig::contig( $bands, $start, $end, $self->{chrom}, $self->{db}, DEFAULT_WIDTH, DEFAULT_HEIGHT, TOP_MARG );

  $self->draw_contig($p_contig);

  return $PNG ? $pic->png : $pic->gif unless $labels;

  #------------------------------  Draw in the band names
  my $lineHeight = gdSmallFont->height * 2;

  # If this is going to be larger than the chromosome, then shrink
  if ( $lineHeight * ($end-$start) > $height ) {
    $lineHeight = $height/($end-$start);
  }
  # If lineHeight <= gdSmallFont then start triaging by removing small and insignificant bands
  my (%exclude,$max_bands);
  if ( $lineHeight < gdSmallFont->height ) {
    $lineHeight = gdSmallFont->height;
    $max_bands = $height/$lineHeight;
    my @parts = sort { $b->[2] - $b->[1] <=> $a->[2] - $a->[1] } @{$bands}[$start..$end];
    foreach ( @parts[$max_bands .. $#parts] ) { $exclude{$_->[0]}++;  }
  }

  my $offset = TOP_MARG + ($lineHeight + $height - $lineHeight * (($max_bands || ($end-$start))+1))/2;
  foreach ( @{$bands}[$start..$end] ) {
    next if $exclude{$_->[0]};
    my $l = $left - H_NAME;
    my $y = $offset + gdSmallFont->height/2;
    my $b = $self->d2p($_->[1] + ($_->[2] - $_->[1])/2);
    
    my $x = $l - gdSmallFont->width * length($_->[0]);
    $pic->string(gdSmallFont,$x,$offset,$_->[0],$black);
    
    $pic->line($l,$y,$l+2,$y,$black);
    $pic->line($l+2,$y,$left-2,$b,$black);
    $pic->line($left-2,$b,$left,$b,$black);
    
    $offset += $lineHeight;
  }

  return $PNG ? $pic->png : $pic->gif;
}

######################################## sub draw_contig
sub draw_contig {
	use GD;
	my $self = shift;
	my $p_contig = shift;
	my %CONTIGS = %{$p_contig};
	my $pic = $self->gd;
	my $brush = GD::Image->new(2,2);
	$brush->colorAllocate(255,255,255);
	my $blue = $brush->colorAllocate(0,0,255);
	$brush->fill(1,1,$blue);
	$pic->setBrush($brush);
	my $black = $pic->colorAllocate(0,0,0);
	my $gray = $pic->colorAllocate(230,230,230);
	for (keys %CONTIGS) {
		my $x = $CONTIGS{$_}{x};
		my $y1 = $CONTIGS{$_}{y1};
		my $y2 = $CONTIGS{$_}{y2};
		my $x_string = $CONTIGS{$_}{x_string};
		my $y_string = $CONTIGS{$_}{y_string};
		my $display_string =  $CONTIGS{$_}{display_string};
		my $display_line = $CONTIGS{$_}{display_line};
#		print "contig <$_>, x <$x>, y1 <$y1>, y2 <$y2>, offset <$offset>\n";
#		$pic->line($x+2,($y1+$y2)/2, $x_string-2, $y_string+4, $gray);
		$pic->line($x, $y1, $x, $y2, gdBrushed) if $display_line;
		$pic->string(gdTinyFont, $x_string, $y_string, $_, $black) if $display_string;

	}
}
##################################################
# subclass of GD for drawing rounded rectangles
##################################################
package GD::Image::Chrom;

use GD; 
use vars '@ISA';
@ISA = 'GD::Image';

# GD is broken for inheritance!
sub new {
  my $class = shift;
  my $image = $class->SUPER::new(@_);
  return bless $image,$class;
}

sub roundrect {
  my $pic = shift;
  my ($left,$top,$right,$bottom,$color,$flat_top,$flat_bottom) = @_;

  # two lines on the left and right
  my $width = $right - $left;
  $pic->line($left,  $flat_top ? $top    : $top+$width/2,
	     $left,$flat_bottom? $bottom : $bottom-$width/2,
	     $color);
  $pic->line($right,   $flat_top ? $top    : $top+$width/2,
	     $right,$flat_bottom ? $bottom : $bottom-$width/2,
	     $color);
    
  # arc at the top
  if ($flat_top) {
    $pic->line($left,$top,$right,$top,$color);
  } else {
    $pic->arc($left+$width/2,$top+$width/2,$width,$width,180,0,$color);
  }

  if ($flat_bottom) {
#    $pic->line($left,$bottom,$right,$bottom,$color);
	$pic->line($left,$bottom-1,$right,$bottom-1,$color);      
  } else {
    $pic->arc($left+$width/2,$bottom-$width/2,$width,$width,0,180,$color);
  }
}

sub fillband {
  my $pic = shift;
  my ($left,$top,$right,$bottom,$start,$stop,$color,
      $flat_top,$flat_bottom,$top_marg) = @_;
##############################
#  print "l <$left>, t <$top>, r <$right>, b <$bottom>, start <$start>, stop <$stop>, flat_t <$flat_top>, flat_b <$flat_bottom>, marg <$top_marg>\n";
#  return;
  my $radius = ($right-$left)/2;
  my $center = $left + $radius;
  my ($l,$r,@r);

  for my $y ($start,$stop) {
	  if (!$flat_top && $y >= $top && $y < $top + $radius) {
		  my $d = ($top + $radius) - $y;
		  my $offset = int (0.5 + sqrt($radius**2 - $d**2));
		  ($l,$r) = ($center-$offset,$center+$offset);
	  } elsif (!$flat_bottom && $y > $bottom - $radius) {
		  my $d = $y - ($bottom - $radius);
		  my $offset = int (0.5 + sqrt($radius**2 - $d**2));
		  ($l,$r) = ($center-$offset,$center+$offset);
	  } 
	  else {
		  ($l,$r) = ($left,$right);
	  }
	  if ($l < $r && $y >= $top) {
			  $pic->line($l,$y,$r,$y,1);
	  }
  }

  next if ($stop-$start <= 1) ||  $start >= $bottom || $start < $top;
  $start++ if $start == $top && ($stop-$start > 2);
  $pic->fill($center,$start+1,$color);
}


1;
