package SNPSubs;

use strict;
use Ace 1.51;
use CGI qw/:standard *table *TR/;
use GeneSubs qw/ENTREZ/;
use constant BLAST_URL => "../snp/printblast";
require Exporter;

use vars qw(@ISA @EXPORT %STYLES);

@ISA = qw(Exporter);
@EXPORT = qw(snp2position ornament clique2picture color_code_key blast_hits %STYLES);

my %patent_codes = (
		    'a/c'   => 'M',
		    'a/g'   => 'R',
		    'a/t'   => 'W',
		    'c/g'   => 'S',
		    'c/t'   => 'Y',
		    'g/t'   => 'K',
		    'a/c/g'  => 'V',
		    'a/c/t'  => 'H',
		    'a/g/t'  => 'D',
		    'c/g/t'  => 'B',
		    'a/c/g/t' => 'N',
		    );

%STYLES = (
	   SNP     => 'Background-color: red;  Color: yellow;',
	   OTHER   => 'Background-color: blue; Color: yellow;',
	   REPEAT  => 'Color: gray;',
	   MAP_STS => 'Text-decoration: underline;',
	   CONFIRM_STS => 'Color: blue;',
	  );

# given a SNP, snp2position returns an array of arrays:
# [object0,pos0],[object1,pos1],[object2,pos2],[object3,pos3]
#     Clique          GB            Link            Chrom
sub snp2position {
  my $snp = shift;
  my @result;
  for ( my $pos  = 1,
	my $obj  = $snp,
	my $prev = undef;
	defined $obj;
	($prev,$obj) = ($obj,$obj->S_Parent(2) && $obj->S_Parent(2)->fetch) ) {

    next unless $prev;

    my @children = $obj->get('S_Child'=>2);
    my ($seq)    = grep {$prev eq $_} @children;
    
    
    if (!defined($pos) or !$seq->right) { # we've gotten ourselves into an unmapped region
      undef $pos;
      push @result,[$obj,$pos];
      next;
    }
    my ($l,$r) = $seq->right->row;
    if ($l < $r ) {
      $pos += $l - 1;
    } else {
      $pos = $l - $pos + 1;
    }
    push @result,[$obj,$pos];
  }

  @result;
}

# Takes the clique and optionally the current SNP and returns a marked-up consensus.
# In scalar context, returns just an HTML marked-up version
# In array context, returns  marked-up version, raw consensus sequence, and a hashref containing
# positions of SNPs in clique
sub clique2picture  {
  my $clique = shift;
  my $mysnp  = shift;

  my $consensus  = $clique->DNA->right;  # fetch consensus DNA

  my %snp_positions;
  my (@markups,@annotations);

  for (my $i=10; $i < length $consensus; $i += 10) {
    push (@markups,[' ',$i]);
  }

  # annotate the position of the SNP
  my @snps = $clique->get('SC_SNP');
  foreach (@snps) {
    my ($snp,$position) = $_->row;
    $snp_positions{$snp} = $position;

    my $allele_string = join '/',sort $snp->Alleles;
    my $nucleotide_string = $patent_codes{$allele_string};
    substr($consensus,$position-1,1) = $nucleotide_string;

    if ($mysnp and $mysnp eq $snp) {
      push (@annotations,['SNP',$position-1,$position]);
    } else {
      push (@annotations,['OTHER',$position-1,$position]);
      push (@markups,[qq(<a class="DNA" style="$STYLES{OTHER}" href="/db/snp/snp?name=$snp">),$position-1],["</a>",$position]); 
    }
    
  }

  # annotate the position of the STS, if any
  if ( my $sts = $clique->SC_STS(1) ) {
    my ($sts,$start,$end) = $sts->row;
    my $length1 = $sts->Left->DNA(2);
    my $length2 = $sts->Right->DNA(2);
    push @annotations,['MAP_STS',$start-1,$start-1+$length1],
	              ['MAP_STS',$end-$length2,$end];
  }

  foreach ($clique->SC_Confirmation_Primer) {
    my ($orientation,$start,$end) = $_->row;
    ($start,$end) = ($end,$start) if $end < $start;
    push @annotations,['CONFIRM_STS',$start,$end];
  }
  
  # add the repeat mask info
  my @repeatMask = $clique->MaskRegions;

  foreach (@repeatMask) {
    my ($start,$end) = $_->row;
    push (@annotations,['REPEAT',$start-1,$end]);
  }

  my $con = $consensus;
  ornament(\$consensus,\@annotations,\@markups);

  return $consensus unless wantarray;
  return ($consensus,$con,\%snp_positions);
}

# annotation structure is:
# @ann = ( ['LABEL',$start,$end], ['LABEL',$start,$end],...)
# sorted by position please
sub ornament {
  my $string       = shift;
  my $markups      = shift;
  my $other_markup = shift;
  my $sequence = 0;

  # linearize markups into a set of start,/end tags, sorted by position
  my @markups = sort { $a->[1]<=>$b->[1] } map { ([ @{$_}[0,1] ],["/$_->[0]",$_->[2]]) } @$markups;

  my (@stack,%attributes,@tags);
  if ($markups) {
    for my $m (@markups) {
      my ($type,$pos) = @{$m};

      $sequence++;

      if ($type !~ m!^/!) { # dealing with a start tag here
	push @tags,[qq(</SPAN seq=$sequence>),$pos] if keys %attributes;
	$sequence++;
	$attributes{uc $type}++;
	my $aggregate_style = join ' ',map { $STYLES{$_} } keys %attributes;
	push @tags,[qq(<SPAN STYLE="$aggregate_style" seq=$sequence>),$pos];

      } else {   # dealing with an end tag here

	(my $t = $type) =~ s!^/!!;
	delete $attributes{uc $t};
	$sequence++;
	push @tags,[qq(</SPAN seq=$sequence>),$pos];
	if (keys %attributes) {
	  $sequence++;
	  my $aggregate_style = join ' ',map { $STYLES{$_} } keys %attributes;
	  push @tags,[qq(<SPAN STYLE="$aggregate_style" seq=$sequence>),$pos];
	  
	}
      }
    }
  }

  push (@tags,@$other_markup) if $other_markup;
  markup($string,\@tags);
}

sub by_pos {
  return $b->[1]<=>$a->[1] unless "$b->[1]" == "$a->[1]";
  my ($seqa) = $a->[0]=~/seq=(\d+)/;
  my ($seqb) = $b->[0]=~/seq=(\d+)/;
  return $seqb <=> $seqa;
}

sub markup {
  my $string = shift;
#line 181
  my $markups = shift;
#  for my $m (sort { $b->[1]<=>$a->[1] or ($a->[0]=~m!^</!)<=>($b->[0]=~m!^</!)} @$markups) {
  for my $m (sort by_pos @$markups) {
    my ($thing,$position) = @$m;
    substr($$string,$position,0) = $thing;
  }
}

# return the key to the consensus sequence
sub color_code_key {
  my @keys = sort keys %patent_codes;
  my $cols = 2;
  my $rows = int (0.99 + @keys / $cols);
  my @extra = (
	       span({-style => $STYLES{SNP}},'This Polymorphism'),
	       span({-style => $STYLES{OTHER}},'Other Polymorphism (link)'),
	       span({-style => $STYLES{REPEAT}},'Repeat masked region'),
	       span({-style => $STYLES{MAP_STS}},'Mapping Primer'),
	       span({-style => $STYLES{CONFIRM_STS}},'Confirmation Primer'),
	       );
  my $table;
  $table .= join '',p,start_table({-border=>undef,-width=>'75%',-align=>'CENTER'}),caption(b('Key')); 
  $table .= TR(th('Ornamentation'),th({-colspan=>6},'Ambiguity Codes'));
  for (my $row = 0; $row < $rows; $row++) {
    $table .= start_TR;
    $table .= td(shift @extra or '&nbsp;');
    for (my $col = 0; $col < $cols; $col++) {
      $table .= join '',th(font({-color=>'red'},$patent_codes{$keys[$row*$cols+$col]}||'&nbsp;')),
                        td($keys[$row*$cols+$col]||'&nbsp;' );
    }
    $table .= end_TR;
  } 
  $table .= end_table;

  return $table;
}

sub blast_hits {
  my $clique = shift;

  my $result;

  if (my @hits = $clique->Homologies) {
    my @rows;
    foreach (@hits) {
      my ($accession,$score,$pval) = $_->row;
      my $object = $accession->fetch;
      (my $gb = $accession) =~s/^\w+://;
      $gb =~ s/\.\d+$//;  # get rid of version #
      my $entrez_acc = $accession;
      $entrez_acc =~ s/^gb://;
      push(@rows,td([a({-href=>ENTREZ . $entrez_acc},$accession),
		     $object->Description,
		     a({-href=>BLAST_URL . "?f=$clique#$gb"},"$clique.blast"),
		     sprintf("%d",$score),
		     sprintf("%0.2e",$pval)]));
    }
    $result = table({-border=>undef},
		TR(th(['Accession#','Description','BLAST File','Score','Pval'])),
		TR(\@rows))
  } else {
    $result = font({-color=>'red'},'No significant BLAST hits');
  }
  return $result;
}

1;
