package Band;
use strict;

use vars '$VERSION';
$VERSION = '1.00';

=pod

=head1 NAME

Band - Simple Cytogenetic Band Class

=head1 SYNOPSIS

 use Band;
 $a = new Band '3q26.32';
 $b = new Band '3p';
 print "$b is less than $a" if $b < $a;

=head1 DESCRIPTION

This is a simple overloaded class suitable for dealing with
cytogenetic coordinates, which have odd ordering rules.  Numeric
comparison (<, >, <=>) is overloaded.  String comparison (cmp, eq, lt,
gt) is not.

=cut

use overload 
  '""'   => 'asString',
  '<=>'  => 'compare';

=pod

=head1 Object Construction

All object construction is through the new() class method.

=over 4

=item $band = Band->new($string)

=item $band = Band->new($band)

new() accept either a string describing the cytogenetic band or
another Band object.  It tests the string for the following band
patterns:

  [\dXY]{1,2}
  [\dXY]{1,2}[pq]
  [\dXY]{1,2}[pq]\d{1,2}
  [\dXY]{1,2}[pq]\d{1,2}\.\d{1,2}

Hence it can detect and properly handle "11", "11q", "11q12",
"11q12.1", and so forth.  If successful, new() returns a new Band
object.  Otherwise it returns undef.

=back

=cut

sub new {
  my $class = shift;
  my $name = shift;

  $class = ref($class) if ref($class);
  my @components;

  if (ref($name) and $name->isa($class)) {
    my @components = @{$name}; # do a copy
  } else {
    @components = $name=~/^([\dXYxy]{1,2})(?:([pq])(?:(\d{1,2})(?:\.(\d{1,2}))?)?)?$/; # chrom, p/q, major, minor
    return unless @components;
    $components[0] = uc $components[0];
    $components[1] = lc $components[1];
  }

  return bless \@components,$class;
}

=pod

=head1 Object Methods

The following object methods give access to the parsed components of
the cytogenetic band:

=over 4

=item $c = $band->chromosome([$chrom])

Return the chromosome.  Can also be used to change the chromosome.

=cut

sub chromosome {
  return defined $_[1] ? $_[0]->[0] = $_[1] : $_[0]->[0];
}

=pod

=item $c = $band->chrom

As per chromosome(), except returns numeric 98 for the X chromosome,
and 99 for the Y chromosome.  Used internally for sorting.  This
cannot be used to change the chromosome.

=cut

sub chrom {
  my $c = $_[0]->chromosome;
  return '98' if $c eq 'X';
  return '99' if $c eq 'Y';
  return $c;
}


=pod

=item $a = $band->arm([$arm])

Gets or sets the chromosome arm ('p' or 'q').

=cut


sub arm {
  return defined $_[1] ? $_[0]->[1] = $_[1] : $_[0]->[1];
}

=pod

=item $a = $band->major([$integer])

Gets or sets the major band number.

=item $a = $band->minor([$integer])

Gets or sets the minor band number.

=cut

sub major {
  return defined $_[1] ? $_[0]->[2] = $_[1] : $_[0]->[2];
}

sub minor {
  return defined $_[1] ? $_[0]->[3] = $_[1] : $_[0]->[3];
}

=pod

=item $a = $band->asString

Returns the string representation of the band.  Used internally to
overload the "" operator.

=cut

sub asString {
  my $s = $_[0]->[0];
  return $s unless my $arm = $_[0]->[1];
  $s .= $arm;
  return $s unless my $major = $_[0]->[2];
  $s .= $major;
  return $s unless my $minor = $_[0]->[3];
  $s .= ".$minor";
}

=pod

=item $order = $band_a->compare($band_b)

Compares two Band objects and returns -1, 0 or 1 to indicate that
C<$band_a> is less than, equal to, or greater than C<$band_b>
respectively.  Used internally to overload the "<=>" operator.

=back

=cut

sub compare {

  my ($self,$other,$reversed) = @_;
  $other = $self->new($other) unless ref($other) and $other->can('chromosome');
  return 0 if $self->asString eq $other->asString;

  ($self,$other) = ($other,$self) if $reversed;

  my $result = $self->chrom <=> $other->chrom || $self->arm cmp $other->arm;
  return $result if $result;

  my $subcomp = $self->major <=>  $other->major || $self->minor cmp $other->minor;
  return $self->arm eq 'p' ? -$subcomp : $subcomp;
}

=pod

=head1 AUTHOR

Lincoln Stein <lstein@cshl.org>.

Copyright (c) 1999 Cold Spring Harbor Laboratory

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.  See DISCLAIMER.txt for
disclaimers of warranty.

=cut

1;
